#!/usr/bin/env python  
# coding=utf-8  
# Python 2.7.3  
from __future__ import print_function

import os
import sys
import argparse
import dataparse
import tempfile
import shutil
import json

from PIL import Image

usage = """
%(prog)s ../btn.plist
%(prog)s ../btn.plist -i ../btn.png
%(prog)s ../data
%(prog)s ../data -r
"""

pvr_file_ext = (".pvr", ".pvr.gz", ".pvr.ccz")
support_file_ext = (".png", ".jpg", ) + pvr_file_ext

logger = None


def log(text):
    if logger:
        logger(text)
    else:
        print(text)    

def get_image_ext(image_file):
    for ext in pvr_file_ext:
        if image_file.endswith(ext):
            return ext
    return os.path.splitext(image_file)[1]

def convert_pvr_to_png(image_file):
    temp_dir = tempfile.mkdtemp()

    shutil.copyfile(image_file, os.path.join(temp_dir, os.path.basename(image_file)))
    image_path = os.path.join(temp_dir, "temp.png")
    plist_path = os.path.join(temp_dir, "temp.plist")

    if os.system("TexturePacker {temp_dir} --sheet {image_path} --texture-format png --border-padding 0 --shape-padding 0 --disable-rotation --allow-free-size --no-trim --data {plist_path}".format(temp_dir = temp_dir, image_path = image_path, plist_path = plist_path)) == 0:
        return image_path

    return None

def unpacker(data_file, image_file=None, output_dir=None, config=None, extra_data_receiver=None):
    # parse file
    data = dataparse.parse_file(data_file, config, extra_data_receiver)
    frame_data_list = data.get("frames") if data else None
    if not data or not frame_data_list:
        log("fail: unknown file type:" + data_file)
        return -1

    # check imagefile
    if not image_file:
        file_path,_ = os.path.split(data_file)
        image_file = os.path.join(file_path , data["texture"])

    # check image format
    image_ext = get_image_ext(image_file)
    if image_ext in pvr_file_ext:
        new_image_file = convert_pvr_to_png(image_file)
        if new_image_file:
            image_file = new_image_file
        else:
            log("fail: can't convert pvr to png, are you sure installed TexturePacker command line tools ? More infomation:\nhttps://www.codeandweb.com/texturepacker/documentation#install-command-line")
            return -1

    # create output dir
    if not output_dir:
        output_dir,_ = os.path.splitext(data_file)
    if not os.path.isdir(output_dir):
        if os.path.exists(os.path.dirname(output_dir)):
            os.mkdir(output_dir)
        else:
            os.makedirs(output_dir)

    try:
        src_image = Image.open(image_file)
    except Exception:
        log("fail: can't open image %s " %image_file)
        return -1

    for frame_data in frame_data_list:
        temp_image = src_image.crop(frame_data["src_rect"])
        if frame_data["rotated"]:
            temp_image = temp_image.rotate(90, expand=1)

        # create dst image
        dst_image = Image.new(src_image.mode, frame_data["source_size"], (0,0,0,0))
        dst_image.paste(temp_image, frame_data["offset"], mask=0)

        output_path = os.path.join(output_dir, frame_data["name"])
        if not os.path.exists(os.path.dirname(output_path)):
            os.makedirs(os.path.dirname(output_path))
        dst_image.save(output_path)

    log("success:" + data_file)
    return 0

# Get the all files & directories in the specified directory (path).
def unpacker_dir(path, recursive, output_dir=None, output=None):
    if output == None:
        output = list()

    for name in os.listdir(path):
        full_name = os.path.join(path, name)
        pre,ext = os.path.splitext(name)
        if ext in ('.plist', ".fnt"):
            output.append(full_name)
            unpacker(full_name, output_dir=os.path.join(output_dir, pre) if output_dir else None)
        elif recursive and os.path.isdir(full_name):
            unpacker_dir(full_name, recursive, os.path.join(output_dir, name) if output_dir else None, output)

    return output
    
def gui():
    try:
        import Tkinter as tk
        import ttk
        import tkFileDialog
    except Exception:
        import tkinter as tk
        import tkinter.ttk as ttk
        import tkinter.filedialog as tkFileDialog
        

    class Application(tk.Frame):
        def __init__(self, root=None):
            tk.Frame.__init__(self, root)
            self.root = root
            self.last_path = None
            root.title("untp")
            self.pack(fill="both", expand=True)
            self.createWidgets()
            self.center()

        def createWidgets(self):

            frame = tk.Frame(self)
            for row in range(0, 2):
                tk.Grid.rowconfigure(frame, row, weight=1)
            for col in range(0, 2):
                tk.Grid.columnconfigure(frame, col, weight=1)
            tk.Button(frame, width=20, text="Unpack Files", command=self.select_file).grid(row=0, column=0, sticky=("N", "S", "E", "W"))
            tk.Button(frame, width=20, text="Unpack Directory", command=self.select_directory).grid(row=0, column=1, sticky=("N", "S", "E", "W"))
            self.recursive_var = tk.IntVar(0)
            self.recursive_var.set(1)
            tk.Checkbutton(frame, text="Recursive", variable=self.recursive_var).grid(row=1, column=1, sticky=tk.W)
            frame.pack(fill="x")

            ttk.Separator(self).pack(fill="x")

            frame = tk.Frame(self)
            scrollbar = tk.Scrollbar(frame) 
            self.logger = tk.Text(frame, width=0, height=5, wrap="word", yscrollcommand=scrollbar.set, borderwidth=0, highlightthickness=0)
            self.logger.bind("<1>", lambda event: self.logger.focus_set())
            scrollbar.config(command=self.logger.yview)
            self.logger.pack(side="left", fill="both", expand=True)
            scrollbar.pack(side="right", fill="y")
            frame.pack(fill="both", expand=True)

        def log(self, text):
            self.logger.config(state='normal')
            self.logger.insert(tk.END, text + '\n')
            self.logger.see(tk.END)  # Scroll to the bottom
            self.logger.config(state='disabled')

        def center(self):
            self.root.update_idletasks()
            w = self.root.winfo_screenwidth()
            h = self.root.winfo_screenheight()
            size = tuple(int(_) for _ in self.root.geometry().split('+')[0].split('x'))
            x = w/2 - size[0]/2
            y = h/2 - size[1]/2
            self.root.geometry("%dx%d+%d+%d" % (size + (x, y)))            

        def select_directory(self):
            path =tkFileDialog.askdirectory()
            if not path:
                return
            self.last_path = path
            unpacker_dir(path, self.recursive_var.get() == 1)

        def select_file(self):
            path = tkFileDialog.askopenfilenames(initialdir=self.last_path)
            file_list = self.root.tk.splitlist(path)
            if not file_list:
                return
            
            self.last_path = os.path.dirname(file_list[0])

            for v in file_list:
                if v.endswith(".plist"):
                    pre,_ = os.path.splitext(v)
                    image_file = None
                    for ext in support_file_ext:
                        if (pre + ext) in file_list:
                            image_file = pre + ext

                    unpacker(v, image_file = image_file)

    root = tk.Tk()
    app = Application(root)

    global logger
    logger = app.log    

    from os import system
    from platform import system as platform
    if platform() == 'Darwin':  # How Mac OS X is identified by Python
        system('''/usr/bin/osascript -e 'tell app "Finder" to set frontmost of process "Python" to true' ''')

    app.mainloop()
    try:
        root.destroy()
    except Exception:
        pass

def main():

    parser = argparse.ArgumentParser(prog="untp", usage=usage)
    parser.add_argument("path", type=str, help="plist/fnt file name or directory")
    parser.add_argument("-o", "--output", type=str, metavar="output", help="specified output directory")

    group_file = parser.add_argument_group('For file')
    group_file.add_argument("-i", "--image_file", type=str, metavar="image_file", help="specified image file for plist")

    group_dir = parser.add_argument_group('For directory')
    group_dir.add_argument("-r", "--recursive", action="store_true", default=False)

    if len(sys.argv) > 1:
        argument = parser.parse_args()

        if os.path.isdir(argument.path):
            return unpacker_dir(argument.path, argument.recursive, output_dir = argument.output)
        elif os.path.isfile(argument.path):
            return unpacker(argument.path, image_file = argument.image_file, output_dir = argument.output)
    else:
        gui()

if __name__ == '__main__':
    main()


#!/usr/bin/env python  
# coding=utf-8  
# Python 2.7.3  

import os
import json
from parse import parse
from plistlib import readPlist
from pprint import pprint

def parse_file(_filepath, _config=None, _extra_data_receiver=None):
	path,name = os.path.split(_filepath)
	pre,ext = os.path.splitext(name)
	if ext == ".plist":
		try:
			data = readPlist(_filepath)
		except Exception:
			print("fail: read plist file failed >", _filepath)
			return
		return parse_plistdata(data)
	elif ext == ".fnt":
		with open(_filepath, "r") as f:
			data = f.read().split("\n")
			if len(data) < 5:
				print("fail: read plist file failed >", _filepath)
				return

		return parse_fntdata(data, _config if _config else {"prefix": pre}, _extra_data_receiver)



def parse_fntdata(_data, _config, _extra_data_receiver=None):
	"""
	info face="Haettenschweiler" size=60 bold=0 italic=0 charset="" unicode=0 stretchH=100 smooth=1 aa=1 padding=0,0,0,0 spacing=2,2
	common lineHeight=64 base=53 scaleW=256 scaleH=128 pages=1 packed=0
	page id=0 file="attack_num.png"
	chars count=12
	char id=52 x=2 y=2 width=33 height=51 xoffset=0 yoffset=5 xadvance=32 page=0 chnl=0 letter="4"
	char id=48 x=37 y=2 width=29 height=50 xoffset=1 yoffset=6 xadvance=29 page=0 chnl=0 letter="0"
	char id=53 x=68 y=2 width=29 height=50 xoffset=1 yoffset=6 xadvance=28 page=0 chnl=0 letter="5"
	char id=57 x=99 y=2 width=28 height=50 xoffset=1 yoffset=6 xadvance=28 page=0 chnl=0 letter="9"
	char id=54 x=129 y=2 width=28 height=50 xoffset=1 yoffset=6 xadvance=28 page=0 chnl=0 letter="6"
	char id=56 x=159 y=2 width=28 height=50 xoffset=1 yoffset=6 xadvance=28 page=0 chnl=0 letter="8"
	char id=51 x=189 y=2 width=28 height=50 xoffset=1 yoffset=6 xadvance=28 page=0 chnl=0 letter="3"
	char id=50 x=219 y=2 width=28 height=49 xoffset=1 yoffset=7 xadvance=28 page=0 chnl=0 letter="2"
	char id=55 x=2 y=55 width=30 height=48 xoffset=1 yoffset=8 xadvance=28 page=0 chnl=0 letter="7"
	char id=49 x=34 y=55 width=20 height=48 xoffset=1 yoffset=8 xadvance=20 page=0 chnl=0 letter="1"
	char id=45 x=56 y=55 width=18 height=12 xoffset=1 yoffset=36 xadvance=19 page=0 chnl=0 letter="-"
	char id=32 x=76 y=55 width=0 height=0 xoffset=11 yoffset=73 xadvance=16 page=0 chnl=0 letter="space"
	"""
	data = {}
	frame_data_list = []

	parse_common_info = parse("common lineHeight={line_height:d} base={base:d} scaleW={scale_w:d} scaleH={scale_h:d} pages={pages:d} packed={packed:d}", _data[1])
	parse_page_info = parse("page id={id:d} file=\"{file}\"", _data[2])
	parse_char_count = parse("chars count={count:d}", _data[3])
	raw_frames_data = {}
	for index in xrange(0, parse_char_count["count"]):
		parse_frame = parse("char id={id:d} x={x:d} y={y:d} width={width:d} height={height:d} xoffset={xoffset:d} yoffset={yoffset:d} xadvance={xadvance:d} page={page:d} chnl={chnl:d} letter=\"{letter}\"", _data[index + 4])

		frame_data = {}
		frame_data["name"]        = "{prefix}_{id}.png".format(prefix= _config["prefix"], id=parse_frame["id"], letter=parse_frame["letter"])
		frame_data["source_size"] = (parse_frame["width"], parse_frame["height"])
		frame_data["rotated"]     = False
		frame_data["src_rect"]    = (parse_frame["x"], parse_frame["y"], parse_frame["x"] + parse_frame["width"], parse_frame["y"] + parse_frame["height"])
		frame_data["offset"]      = (0, 0)

		if parse_frame["width"] <= 0 or  parse_frame["height"] <= 0:
			continue

		frame_data_list.append(frame_data)

		parse_frame_named_data = parse_frame.named.copy()
		parse_frame_named_data["texture"] = frame_data["name"]

		raw_frames_data[parse_frame["id"]] = parse_frame_named_data

	data["texture"] = parse_page_info["file"]
	data["frames"] = frame_data_list

	if _extra_data_receiver != None:
		_extra_data_receiver["common"] = parse_common_info.named
		_extra_data_receiver["frames"] = raw_frames_data

	return data


def _mapping_list(_result, _name, _data):
	for i,v in enumerate(_name):
		if isinstance(v, list):
			_mapping_list(_result, v, _data[i])
		else:
			_result[v] = _data[i]

	return _result

def _parse_str(_name, _str):
	return _mapping_list({}, _name, json.loads(_str.replace("{", "[").replace("}", "]")))

def parse_plistdata(_data):
	fmt = _data.metadata.format
	# check file format
	if fmt not in (0, 1, 2, 3):
		print("fail: unsupport format " + str(fmt))
		return None

	data = {}
	frame_data_list = []

	for (name,config) in _data.frames.items():
		frame_data = {}
		if fmt == 0:
			source_size = {
				"w": config.get("originalWidth", 0),
				"h": config.get("originalHeight", 0),
			}
			rotated = False
			src_rect = (
				config.get("x", 0),
				config.get("y", 0),
				config.get("x", 0) + config.get("originalWidth", 0),
				config.get("y", 0) + config.get("originalHeight", 0),
			)
			offset = {
				"x": config.get("offsetX", False),
				"y": config.get("offsetY", False),
			}
		elif fmt == 1 or fmt == 2:
			frame         = _parse_str([["x","y"],["w","h"]], config.frame)
			center_offset = _parse_str(["x","y"], config.offset)
			source_size   = _parse_str(["w","h"], config.sourceSize)
			rotated       = config.get("rotated", False)
			src_rect      = (
				frame["x"],
				frame["y"],
				frame["x"]+(frame["h"] if rotated else frame["w"]),
				frame["y"]+(frame["w"] if rotated else frame["h"])
			)			
			offset = {
				"x": source_size["w"]/2 + center_offset["x"] - frame["w"]/2, 
				"y": source_size["h"]/2 - center_offset["y"] - frame["h"]/2,
			}
		elif fmt == 3:
			frame         = _parse_str([["x","y"],["w","h"]], config.textureRect)
			center_offset = _parse_str(["x","y"], config.spriteOffset)
			source_size   = _parse_str(["w","h"], config.spriteSourceSize)
			rotated       = config.textureRotated
			src_rect      = (
				frame["x"],
				frame["y"],
				frame["x"]+(frame["h"] if rotated else frame["w"]),
				frame["y"]+(frame["w"] if rotated else frame["h"])
			)			
			offset = {
				"x": source_size["w"]/2 + center_offset["x"] - frame["w"]/2, 
				"y": source_size["h"]/2 - center_offset["y"] - frame["h"]/2,
			}
		else:
			continue

		frame_data["name"]        = name
		frame_data["source_size"] = (int(source_size["w"]), int(source_size["h"]))
		frame_data["rotated"]     = rotated
		frame_data["src_rect"]    = [int(x) for x in src_rect ]
		frame_data["offset"]      = (int(offset["x"]), int(offset["y"]))

		frame_data_list.append(frame_data)


	data["frames"] = frame_data_list
	data["texture"] = _data.metadata.textureFileName

	return data


#!/usr/bin/env python

from setuptools import setup, find_packages

setup(
    name = 'untp',
    version = '1.1.3',
    description = 'A command line tool to split TexturePacker published files.',
    url = 'https://github.com/justbilt/untp',
    author = 'justbilt',
    author_email = 'wangbilt@gmail.com',
    license = 'MIT License',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],    
    keywords = 'untp texturepacker cocos',
    install_requires = [
        'Pillow',
        'parse'
    ],
    packages = find_packages("src"),
    package_dir = {'':'src'},
    entry_points = {
        'console_scripts': [
            'untp = untp.untp:main',
        ],
    }
)